#version 330 core
#define PId3 3.14159265 / 3

layout (points) in;
layout (triangle_strip, max_vertices = 36) out;

in VertOut {
	vec3 colorTint;
	int sides;
} vertOut[];

out FragIn {
	vec2 texCoords;
	vec3 colorTint;
} fragIn;

uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform float radius;
uniform float camX;


void main() {
	fragIn.colorTint = vertOut[0].colorTint;
	fragIn.texCoords = vec2(0, 0);
	mat4 matrix = projectionMatrix * viewMatrix;
	vec4 pos = gl_in[0].gl_Position;
	int sides = vertOut[0].sides;
	
	vec3 pos_map[12];
	vec2 tex_map[6];
	int index_map[6];
	
	float y60 = sin(PId3), x60 = cos(PId3);
	float angleHalfHexagon = y60 / radius; // radius / y60 = 2 * ringSize * chunkSize / (2 * PI)
	float mult1 = pow(1 - 1 / radius, pos.y), mult2 = pow(1 - 1 / radius, pos.y + 1);
	float xpos = (x60 + 1) * pos.x;
	
	for (int i = 1, idx = 0; i > -2; i--) { // 1, 0, -1
		float v1 = ((pos.x + 2 * pos.z) + i) * angleHalfHexagon;
		float v2 = PId3 * i;
		float y = sin(v1) * radius, z = cos(v1) * radius, cosv2 = cos(v2), sinv2 = sin(v2);
		
		int index = (6 + i) % 6;
		
		index_map[idx] = index;
		tex_map[index] = vec2(cosv2, sinv2);
		pos_map[index] = vec3(xpos + cosv2 - camX, y, z) * mult1;
		pos_map[index].x += camX;
		pos_map[index + 6] = vec3(xpos + cosv2 - camX, y, z) * mult2;
		pos_map[index + 6].x += camX;
		
		index = (6 + 3 - i) % 6;
		
		index_map[idx + 1] = index;
		tex_map[index] = vec2(-cosv2, sinv2);
		pos_map[index] = vec3(xpos - cosv2 - camX, y, z) * mult1;
		pos_map[index].x += camX;
		pos_map[index + 6] = vec3(xpos - cosv2 - camX, y, z) * mult2;
		pos_map[index + 6].x += camX;
		idx += 2;
	}
	
	if ((sides & 1) != 0) {
		for (int i = 0; i < 6; i++) {
			int index = index_map[i];
			gl_Position = matrix * vec4(pos_map[index], 1);
			fragIn.texCoords.x = (1 + tex_map[index].x) / 2;
			fragIn.texCoords.y = (1 - tex_map[index].y) / 2;
			EmitVertex();
		}
		EndPrimitive();
	}
	if ((sides & 1 << 1) != 0) {
		for (int i = 0; i < 6; i++) {
			int i2 = index_map[i];
			int i1 = index_map[5 - i] + 6;
			gl_Position = matrix * vec4(pos_map[i1], 1);
			fragIn.texCoords.x = (1 - tex_map[i2].x) / 2;
			fragIn.texCoords.y = (1 - tex_map[i2].y) / 2;
			EmitVertex();
		}
		EndPrimitive();
	}
	for (int j = 0; j < 6; j++) {
		if ((sides & 1 << j + 2) != 0) {
			for (int i = 0; i < 4; i++) {
				int index = (j + 1 - i % 2) % 6 + 6 * (i / 2);
				gl_Position = matrix * vec4(pos_map[index], 1);
				fragIn.texCoords.x = 1 - i % 2;
				fragIn.texCoords.y = i / 2;
				EmitVertex();
			}
			EndPrimitive();
		}
	}
}